// commands.cpp - implementation of each console command 
//
// see LICENSE.txt for licensing information
//
// (c) 2008 Wesley M. Eddy
// 

#include "commands.h"

const wchar_t *promptString1 = L"saratoga> ";

void exitCmdHandler(std::wstring &rest) {
	wprintf(L"exiting ...\n");
	killAllThreads = true;
}

void helpCmdHandler(std::wstring &rest) {
	wprintf(L"valid commands are:\n");
	unsigned short i;
	for (i = 0; i < MAX_CMDS; i++) {
		wprintf(cmds[i].helper.c_str());
		wprintf(L"\n");
	}
}

void eidCmdHandler(std::wstring &rest) {
	if (rest.length() == 0) {
		wprintf(saratoga::localEID.c_str());
		wprintf(L"\n");
	}
	else saratoga::setLocalEID(rest);
}

void peersCmdHandler(std::wstring &rest) {
	std::list<std::wstring> l;
	saratoga::listPeers(l);
	std::list<std::wstring>::iterator i;
	size_t n = 1;
	wprintf(L"%d peers:\n", l.size());
	for (i = l.begin(); i != l.end(); i++, n++) {
		wprintf(L"%d) EID: %ls\n", n, i->c_str());
	}
}

void peerCmdHandler(std::wstring &rest) {
	size_t eidEnd, ipEnd;
	eidEnd = rest.find(L" ");
	if (eidEnd == std::wstring::npos) {
		wprintf(L"no IP address and port given!\n");
		return;
	}
	ipEnd = rest.find(L" ", eidEnd+1);
	if (ipEnd == std::wstring::npos)
	{
		wprintf(L"no port given\n");
		return;
	}

	std::wstring eid = rest.substr(0, eidEnd).c_str();
    std::wstring wIpAddr = rest.substr(eidEnd+1, (ipEnd-eidEnd)-1);
	std::string ipAddr = saratoga::stringFromWstring(wIpAddr.c_str(), wIpAddr.length());
	std::string portStr = saratoga::stringFromWstring(rest.substr(ipEnd+1).c_str(), rest.length()-(ipEnd+1));
    uint16_t port = atoi(portStr.c_str());
	saratoga::addPeer(eid, ipAddr, port);
}

std::wstring filePart(std::wstring &path) {
    std::wstring sep;
#ifdef _WIN32
    sep = L"\\";
#else
    sep = L"/";
#endif
    size_t sepIndex = path.find_last_of(sep);
    if (sepIndex == std::wstring::npos) sepIndex = 0;
    else sepIndex++; // advance past path separator
    return path.substr(sepIndex);
}

void getCmdHandler(std::wstring &rest) {
	std::wstring peer, localFile, remoteFile;
	size_t peerEidEnd, remoteFileEnd;
	peerEidEnd = rest.find(L" ");
	if (peerEidEnd == std::wstring::npos) {
		wprintf(L"no remote file given!\n");
		return;
	}
	peer = rest.substr(0, peerEidEnd).c_str();
    rest = rest.substr(peerEidEnd+1);
	remoteFileEnd = rest.find(L" ");
	if (remoteFileEnd == std::wstring::npos) {
		remoteFile = rest.c_str();
		localFile = filePart(remoteFile);
	} else {
		remoteFile = rest.substr(0, remoteFileEnd);
        localFile = rest.substr(remoteFileEnd+1);
	}

	// start GET transaction
	saratoga::startGetTransaction(&peer, &remoteFile, &localFile);
}

void getdirCmdHandler(std::wstring &rest) {
	// XXX
	wprintf(L"not in this version of code\n");
}

void putCmdHandler(std::wstring &rest) {
	// XXX
	wprintf(L"not in this version of code\n");
}

void deleteCmdHandler(std::wstring &rest) {
	// XXX
	wprintf(L"not in this version of code\n");
}

void serveCmdHandler(std::wstring &rest) {
	saratoga::serveRequests = true;
}

void noServeCmdHandler(std::wstring &rest) {
	saratoga::serveRequests = false;
}

void beaconCmdHandler(std::wstring &rest) {
	saratoga::sendBeaconPackets = true;
}

void noBeaconCmdHandler(std::wstring &rest) {
	saratoga::sendBeaconPackets = false;
}

void setPortCmdHandler(std::wstring &rest) {
	saratoga::cleanup();
    std::string portStr = saratoga::stringFromWstring(rest.c_str(), rest.length());
	uint16_t localPort = atoi(portStr.c_str());
	saratoga::localPort = localPort;
	saratoga::startup();
}

cmdHandler cmds[MAX_CMDS] = {
	{ std::wstring(L"exit"), &exitCmdHandler,
		L"exit   -   leave the application" },
	{ std::wstring(L"quit"), &exitCmdHandler,
		L"quit   -   leave the application" },
	{ std::wstring(L"help"), &helpCmdHandler,
		L"help   -   print this help message" },
	{ std::wstring(L"eid"), &eidCmdHandler,
		L"eid [newEID]   -   show local EID or set to newEID" },
	{ std::wstring(L"peers"), &peersCmdHandler,
		L"peers   -   show list of known peers from beacons or manual" \
		L" adds" },
	{ std::wstring(L"peer"), &peerCmdHandler,
		L"peer peerEID peerIP peerPort   -   add a peer manually" },
	{ std::wstring(L"getdir"), &getdirCmdHandler,
		L"getdir peerEID remotePath   -   list files on peer" },
	{ std::wstring(L"get"), &getCmdHandler,
		L"get peerEID remoteFile [localFile]   -   download remoteFile from given peer" },
	{ std::wstring(L"put "), &putCmdHandler,
		L"put peerEID localFile remoteFile   -   send a file to peer" },
	{ std::wstring(L"delete "), &deleteCmdHandler,
		L"delete peerEID remoteFile   -   remove file from peer" },
	{ std::wstring(L"serve"), &serveCmdHandler,
		L"serve   -   serve files from working directory" },
	{ std::wstring(L"no serve"), &noServeCmdHandler,
		L"no serve   -   stop serving files locally" },
	{ std::wstring(L"beacon"), &beaconCmdHandler,
		L"beacon   -   send beacons out for peers to see" },
	{ std::wstring(L"no beacon"), &noBeaconCmdHandler,
		L"no beacon   -   stop sending beacons" },
	{ std::wstring(L"port"), &setPortCmdHandler,
		L"port N   -   set local port number used to N" }
};

void handleCommand(std::wstring &c) {
	unsigned short i;
	bool handled = false;
    std::wstring empty = L"";
	for (i = 0; i < MAX_CMDS && !handled; i++) {
		unsigned short j = c.find(cmds[i].cmd);
		if (j == 0) {
            size_t beginRest = cmds[i].cmd.length()+1;
            std::wstring rest = beginRest < c.length() ?
                                c.substr(cmds[i].cmd.length()+1) :
                                L"";
			cmds[i].handler(c.length() <=
				cmds[i].cmd.length()+1 ? empty : rest);
			handled = true;
		}
	}
	if (!handled) helpCmdHandler(empty);
	if (!killAllThreads) wprintf(promptString1);
	return;
}
